Type: Package
Package: rprimer
Title: Design Degenerate Oligos from a Multiple DNA Sequence Alignment
Version: 1.17.1
Description: 
    Functions, workflow, and a Shiny application for visualizing sequence 
    conservation and designing degenerate primers, probes, and (RT)-(q/d)PCR 
    assays from a multiple DNA sequence alignment. The results can be presented 
    in data frame format and visualized as dashboard-like plots. For more information, 
    please see the package vignette.
Authors@R: 
    person(given = "Sofia",
           family = "Persson",
           role = c("aut", "cre"),
           email = "sofiapersson27@gmail.com",
           comment = c(ORCID = "0000-0003-2611-3030"))
License: GPL-3
URL: https://github.com/sofpn/rprimer
BugReports: https://github.com/sofpn/rprimer/issues
Depends: R (>= 4.1), Biostrings, MultipleAlignment,
Imports: bslib, DT, ggplot2, IRanges, mathjaxr, methods, patchwork,
        reshape2, S4Vectors, shiny, shinycssloaders, shinyFeedback
Suggests: BiocStyle, covr, kableExtra, knitr, rmarkdown, styler,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Encoding: UTF-8
RdMacros: mathjaxr
biocViews: Alignment, ddPCR, Coverage, MultipleSequenceAlignment,
        SequenceMatching, qPCR
biocType: Software
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/rprimer
git_branch: devel
git_last_commit: 32bb697
git_last_commit_date: 2026-07-29
Repository: Bioconductor 3.24
Date/Publication: 2026-07-30
NeedsCompilation: no
Packaged: 2026-07-31 00:01:25 UTC; biocbuild
Author: Sofia Persson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2611-3030>)
Maintainer: Sofia Persson <sofiapersson27@gmail.com>
