# Generated by roxygen2: do not edit by hand

export(RprimerAssay)
export(RprimerMatchAssay)
export(RprimerMatchOligo)
export(RprimerOligo)
export(RprimerProfile)
export(checkMatch)
export(consensusProfile)
export(designAssays)
export(designOligos)
export(plotData)
export(runRprimerApp)
exportMethods(checkMatch)
exportMethods(plotData)
import(mathjaxr)
import(methods)
import(shiny)
importClassesFrom(Biostrings,DNAStringSet)
importClassesFrom(S4Vectors,DataFrame)
importFrom(MultipleAlignment,colmask)
importFrom(MultipleAlignment,readDNAMultipleAlignment)
importFrom(MultipleAlignment,rowmask)
importFrom(patchwork,wrap_plots)
