#' @details
#' The package contains five functions: \code{consensusProfile()},
#' \code{designOligos()}, \code{designAssays()}, \code{checkMatch()} 
#' and \code{plotData()}
#'
#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL
